# by DCL manual
#english
=begin
==NumRu::AdvancedDCL::Dateaxis
===About this module
The Dateaxis module is a subroutine package for plotting a coordinate axis with dates for tick marks.
To plot this type of coordinate axis, the normalization transformation must be 
set using units in days. The coordinate axis will be plotted from the position 
corresponding to 0 in the user-defined coordinate system (UC). Therefore, if you wish to allot a period from Jan. 1 to Jan. 31 to the full viewport size, specify the ends of the window as (0, 30).

Below are explanation of the arguments CSIDE, JD0, and ND, which are common in the subroutines.
* CSIDE - a character-type argument which specifies the position. For the X axis, specify as 'B'ottom, 'T'op, or 'U'ser; for the Y axis, specify 'L'eft, 'R'ight, or 'U'ser.
* JD0 - the date of the position corresponding to the 0 in UC. The numerals left of the 5th digit from the right represent year, the 3rd and 4th digits from right represent month, and the 1st and 2nd digits from right represent day. For example, Sept. 1, 1990 is expressed as 19900901.
* ND - The number of days to plot when <tt>JD0</tt> is the day 0. For example, if you wish to make a plot for the period between Sept. 1, 1990 to Sept. 30, 1990, then set <tt>JD0</tt> as 19909001 and <tt>ND</tt> as 29.
 
===Parametrs
Name  Type  DCL package
:nday (Fixnum) <UCPACK> 
   Specifies the interval for plotting tick marks and labels of dates when using Dateaxis.xady, Dateaxis.yady. If 'nday' is undefined, then an appropriate value is selected from 0, 1, 2, 5, 10, or 15. (When 0, only the axis is plotted, and no ticks or labels are drawn.) When explicitly specified, the specified value is used. (The initial value is undefined.) This value is specified by the DCL function 'NUCDAY'. 
:nchar (Fixnum) <UCPACK>
   Specifies the number of letters in the month for the label when using Dateaxis.xamn, Dateaxis.yamn. If 'nchar' is undefined, then an appropriate value is selected from 0, 1, 2, 3, or 9. (When 0, only the axis is plotted, and no ticks or months are drawn) When explicitly specified, the specified value is used. (The initial value is undefined.) This value is specified by the DCL function 'NUCCHR'. 
:lower (true or false) <UCPACK>
   Specifies whether to write the letters after the second letter in lower case for the name of the month written by Dateaxis.xady, Dateaxis.yady. In lower case when true; in upper case when false. (The initial value is false.)
:iundef (Fixnum) <UCPACK> 
   The internal variable indicating that the value is undefined. (The initial value is -999.)

In addition, These parameters can be used as methods. For example, it is possible to refer the parameter "nday" with Dateaxis.nday method. Moreover, it is possible to set the parameter "nday" with Dateaxis.nday= method.

=end
#japanese
=begin
==NumRu::AdvancedDCL::Dateaxis
==={W[ɂ
DateaxisW[͓t`߂̃W[łB
悵悤ƂWɂĂ, PʂƂĐKϊݒ肵ĂȂ΂ȂȂ. ܂W̍, [U[Wn(UC)ɂ0ɑʒu炨Ȃ. , Ƃ11131܂31r[|[gςɊ蓖ĂƂ,EChE̗[l(0, 30)Ǝw肷. 

ȉŃTu[`ɋʂĂ CSIDE, JD0, NDɂĐ. 
* CSIDE - ꏊw肷镶^̈. XɊւĂ'B'ottom, 'T'op, 'U'serwł. YɊւĂ'L'eft, 'R'ight, 'U'serwł. ڂAxisTvQƂ̂. 
* JD0 - UC0ɑꏊ̓t. 5ȏオN, 3,4, 1,21œt킷. Ƃ1990N9119900901ƕ\. 
* ND - JD00ڂƂč悷. Ƃ1990N911990N930܂30`Ƃ JD019900901ƂND29Ƃ΂悢.
 
===p^
p^ p^̌^ ΉDCLpbP[W
:nday (Fixnum) <UCPACK> 
   Dateaxis.xady, Dateaxis.yady gƂ,ƂɖڐƓt`w肷. nday sȂ0, 1, 2, 5, 10, 15̒KȒlI΂(0ȉ̂Ƃ͎`ŖڐƓt͕`Ȃ);zɗ^Ă΂̒lg(l͕s).  DCL ̊֐ 'NUCDAY'߂.
:nchar (Fixnum) <UCPACK>
   Dateaxis.xamn, Dateaxis.yamn gƂ,`w肷. ncharsȂ0, 1, 2, 3, 9̒KȒlI΂( 0 ̂Ƃ͎`Ŗڐƌ͕`Ȃ);zɗ^Ă΂̒lg(l͕s). DCL ̊֐ 'NUCCHR' ߂.
:lower (true or false) <UCPACK>
   Dateaxis.xady, Dateaxis.yady ŕ`2ڈȉɂ邩ǂw肷. true Ȃ珬ɂ; false Ȃ啶̂܂܂ɂ(l false).
:iundef (Fixnum) <UCPACK> 
   lsł邱Ƃϐ(l -999).

܂Ãp^̓\bhƂėp邱ƂłBȂ킿ADateaxis.nday Ńp^ nday ̎QƂA Dateaxis.nday= Ńp^ nday ̕ύXłB

=end

module NumRu
  module AdvancedDCL
    module Dateaxis
      def param_init
	@param = Parameter.new(Common.param)
	
	@param.init("nday", Fixnum, "uciset", "uciget", 0, 0)
	@param.init("nchar", Fixnum, "uciset", "uciget", 0, 0)
	@param.init("lower", TrueClass, "uclset", "uclget", 0, 0)
	@param.init("iundef", Fixnum, "uciset", "uciget", 0, 0)
	
	@axisparam = Axis.param
	@temp_param = @param.dup
	@backup = @param.dup
	@orig_param = @param.dup
	@set = Hash.new
      end
      
      private :param_init

#------------------- method for parameters----------------------
      
      include Parameter_appli
      
# by kawanabe
#english
=begin
---Dateaxis.set(key, val)
   Sets parameters. If val is nil, the value of the parameter will return default value in DCL.

   Arguments
   * key: parameter name(String)
   * val: new value. About type, refer each parameter.

   Example
      Dateaxis.set("nday", 15)      # Sets 'nday'15.

=end
#japanese
=begin
---Dateaxis.set(key, val)
    p^̐ݒsBvalnilw肷ƒlDCLɂ鏉lɖ߂B

    
    * key:p^(String)
    * val:ݒ肷lB^͊ep^QƂ̂ƁB
    gp
        Dateaxis.set("nday", 15)      # nday  15 ɐݒ

=end

# by kawanabe
#english
=begin
---Dateaxis.get(key)
    Refers the value of the parameter.

    Arguments
    * key: parameter name(String)

    Example
        Dateaxis.get("nday")      # Refers 'nday'.

=end
#japanese
=begin
---Dateaxis.get(key)
    p^QƂB

    
    * key:p^(String)
    gp
        Dateaxis.get("nday")      # nday QƂB

=end

# by kawanabe
#english
=begin
---Dateaxis.reset(key)
    Sets the value of the parameter default value.

    Arguments
    * key: parameter name(String)

    Example
        Dateaxis.reset("nday")      # Sets 'nday' a default value.

=end
#japanese
=begin
---Dateaxis.reset(key, val)
    p^DCLɂ鏉lɖ߂B

    
    * key:p^(String)
    * val:ݒ肷lB^͊ep^QƂ̂ƁB

    gp
        Dateaxis.reset("nday")      # nday lɖ߂

=end

#english
=begin
---Dateaxis.help_params
    Refers available parameters in Dateaxis module.

    Arguments
    None

    Notes
    (1)This method return name of the parameter, present value, type of the parameter.

=end
#japanese
=begin
---Dateaxis.help_params
    DateaxisŐݒ\ȃp^QƂB

    
    Ȃ

    l
    (1)̃\bh͐ݒ\ȃp^ɊւāAp^A݂̒lAp^̌^ԂB

=end

#english
=begin
---Dateaxis.sync
    Synchronizes the value of a parameter in DCL with the present value of the parameter in AdvancedDCL.

    Arguments
     None

    Notes
     In AdvancedDCL, parameters which set with "set" method do not synchronize with DCL and a method to synchronize parameters is called in drawing methods. This method is used to synchronize parameters with these of DCL directly.

=end 
#japanese
=begin
---Dateaxis.sync
    ݁Aݒ肳Ăp^DCLɔfB

    
     Ȃ

    l
     AdvancedDCLɂĂset\bĥ݂ł̓p^DCLɔfꂸA
p^sync郁\bhŌĂłB̃\bh̓[Up
^zsyncł邽߂̃\bhłB

=end
 

      def xdraw(cside, jd0, nd, x=nil, y=nil, temp=nil)
	if(x.is_a?(Hash))
	  temp = x
	elsif((x.is_a?(NArray))&&(x.rank==2))
	  y = x.shape[1].to_f
	  x = x.shape[0].to_f
	  Basics.checkundef(x, y)
	elsif(Basics.typearray(x) && Basics.typearray(y))
	  Basics.checkundef(x, y)
	elsif(x==nil&&y==nil)
	  Basics.checkundef(1.0, 1.0)
	end
	@param.sync
	@axisparam.sync
	if(temp)
	   @temp_param.set(temp, @param, @backup)
	   @temp_param.sync
	end
	Fig.settrf
	
	DCL.ucxacl(cside, jd0, nd)
	if(temp)
	   @backup.sync
	   @temp_param.clear
	end
	nil
      end

# by DCL manual
#english
=begin
---Dateaxis.xdraw(cside, jd0, nd, x=nil, y=nil, temp=nil)
    Plots a coordinate axis for day, month, and year.

    Arguments
    * cside (String) : Specifies the position to plot the axis.
    * jd0 (Fixnum) : Specifies the date of the position corresponding to 0 in UC.
    * nd (Fixnum) : Specifies the number of days to plot.
    * x, y (Array or NArray) : Arrays for specifying the window.
    * temp (Hash) : temporary parameters

    l
    (1)UCXACL is called in this method.
    (2)Dateaxis.xdraw makes successive calls to Dateaxis.xday, Dateaxis.xamn, Dateaxis.xayr.

=end 
#japanese
=begin
---Dateaxis.xdraw(cside, jd0, nd, x=nil, y=nil, temp=nil)
    EENɊւW`. 

    
    * cside (String) : Wꏊw肷.
    * jd0 (Fixnum) : 0ɑꏊ̓tw肷.
    * nd (Fixnum) : 悷w肷.
    * x, y (Array or NArray) : EBhEݒ肷邽߂XAY̔z
    * temp (Hash) : e|p^

    l
    (1)̃\bh UCXACL ĂłB
    (2)Dateaxis.xdraw  Dateaxis.xady, Dateaxis.xamn, Dateaxis.xayr (ȉQ)𑱂ČĂł.

=end 

      def ydraw(cside, jd0, nd, x=nil, y=nil, temp=nil)
	if(x.is_a?(Hash))
	  temp = x
	elsif((x.is_a?(NArray))&&(x.rank==2))
	  y = x.shape[1].to_f
	  x = x.shape[0].to_f
	  Basics.checkundef(x, y)
	elsif(Basics.typearray(x) && Basics.typearray(y))
	  Basics.checkundef(x, y)
	elsif(x==nil&&y==nil)
	  Basics.checkundef(1.0, 1.0)
	end
	@param.sync
	@axisparam.sync
	if(temp)
	   @temp_param.set(temp, @param, @backup)
	   @temp_param.sync
	end
	Fig.settrf
	
	DCL.ucyacl(cside, jd0, nd)
	if(temp)
	   @backup.sync
	   @temp_param.clear
	end
	nil
      end

# by DCL manual
#english
=begin
---Dateaxis.ydraw(cside, jd0, nd, x=nil, y=nil, temp=nil)
    Plots a coordinate axis for day, month, and year.

    Arguments
    * cside (String) : Specifies the position to plot the axis.
    * jd0 (Fixnum) : Specifies the date of the position corresponding to 0 in UC.
    * nd (Fixnum) : Specifies the number of days to plot.
    * x, y (Array or NArray) : Arrays for specifying the window.
    * temp (Hash) : temporary parameters

    l
    (1)UCYACL is called in this method.
    (2)Dateaxis.ydraw makes successive calls to Dateaxis.yday, Dateaxis.yamn, Dateaxis.yayr.

=end 
#japanese
=begin
---Dateaxis.ydraw(cside, jd0, nd, x=nil, y=nil, temp=nil)
    EENɊւW`. 

    
    * cside (String) : Wꏊw肷.
    * jd0 (Fixnum) : 0ɑꏊ̓tw肷.
    * nd (Fixnum) : 悷w肷.
    * x, y (Array or NArray) : EBhEݒ肷邽߂XAY̔z
    * temp (Hash) : e|p^

    l
    (1)̃\bh UCYACL ĂłB
    (2)Dateaxis.ydraw  Dateaxis.yady, Dateaxis.yamn, Dateaxis.yayr (ȉQ)𑱂ČĂł.

=end 

      def xady(cside, jd0, nd, x=nil, y=nil, temp=nil)
	if(x.is_a?(Hash))
	  temp = x
	elsif((x.is_a?(NArray))&&(x.rank==2))
	  y = x.shape[1].to_f
	  x = x.shape[0].to_f
	  Basics.checkundef(x, y)
	elsif(Basics.typearray(x) && Basics.typearray(y))
	  Basics.checkundef(x, y)
	elsif(x==nil&&y==nil)
	  Basics.checkundef(1.0, 1.0)
	end
	@param.sync
	@axisparam.sync
	if(temp)
	   @temp_param.set(temp, @param, @backup)
	   @temp_param.sync
	end
	Fig.settrf
	
	DCL.ucxady(cside, jd0, nd)
	if(temp)
	   @backup.sync
	   @temp_param.clear
	end
	nil
      end

# by DCL manual
#english
=begin
---Dateaxis.xady(cside, jd0, nd, x=nil, y=nil, temp=nil)
    Plots the coordinate axis for days.

    Arguments
    * cside (String) : Specifies the position to plot the axis.
    * jd0 (Fixnum) : Specifies the date of the position corresponding to 0 in UC.
    * nd (Fixnum) : Specifies the number of days to plot.
    * x, y (Array or NArray) : Arrays for specifying the window.
    * temp (Hash) : temporary parameters

    notes
    (1)UCXADY is called in this method.
    (2)When the window is not specified, the window is specified from x and y if they are specified as arguments.
    (3) The intervals of the tick marks and labels for date are specified using the internal variable 'nday' though the DCL function NUCDAY. When 'nday' is undefined, then an appropriate value is selected out of 0, 1, 2, 5, 10, and 15. (When the value is smaller than 1, only the axis is plotted and no ticks or labels are plotted.) When explicitly specified, the specified value is used. (The initial value is undefined.)
    (4) An array is prepared internally for storing the information for plotting the ticks and days on the labels. The array size restricts the maximum number of ticks and dates (labels) to 100.
    (5) Dateaxis.xady, Dateaxis.yady plots axis using Axis::Lin.xpaxs, Axis::Lin.ypaxs (ISLCT=2), plots tick marks using Axis::Lin.xptmk, Axis::Lin.yptmk (ISLCT=1), and labels of days using Axis::Lin.xplbl, Axis::Lin.yplbl (ISLCT=1). Therefore, the attributes for these component elements of the axes can be inquired/changed using the Axis::Lin module. However, 'icent(zs)' must be 0. 'irotl(zs)' inquires 'irotc(zs)' and uses the value. 'lbtwn' is false. 

=end
#japanese
=begin
---Dateaxis.xady(cside, jd0, nd, x=nil, y=nil, temp=nil)
    ɊւW`. 

    
    * cside (String) : Wꏊw肷.
    * jd0 (Fixnum) : 0ɑꏊ̓tw肷.
    * nd (Fixnum) : 悷w肷.
    * x, y (Array or NArray) : EBhEݒ肷邽߂XAY̔z
    * temp (Hash) : e|p^

    l
    (1)̃\bh UCXADY ĂłB
    (2)EBhEw肵ĂȂƂAƂx,yw肷x,yɃEBhEݒ肵A`B
    (3) ƂɖڐƓt`, DCL ̊֐ NUCDAY ʂăp^ nday ߂. nday sȂ0, 1, 2, 5, 10, 15̒KȒlI΂( 0 ȉ̂Ƃ͎`ŖڐƓt͕`Ȃ); zɗ^Ă΂̒lg(l͕s). 
    (4) ڐƓt`߂̏ێ邽߂ ŔzpӂĂ. ̔z̑傫Ɋւ鐧, `ڐƓt͍ő100ł. 
    (5) Dateaxis.xady, Dateaxis.yady ͂ꂼ Axis::Lin.xpaxs, Axis::Lin.ypaxs (islct = 2)Ŏ`Ă; Axis::Lin.xptmk, Axis::Lin.yptmk (islct = 1)Ŗڐ`Ă; Axis::Lin.xplbl, Axis::Lin.yplbl (islct = 1)œ̃x`Ă. , W̍\vf̑, Axis::Lin ̃p^ɂĎQ/ύX邱Ƃł. , icent(zs)  0 Ƃ; irotl(zs)  irotc(zs)QƂėp; lbtwn  false Ƃ. 

=end
    
      def yady(cside, jd0, nd, x=nil, y=nil, temp=nil)
	if(x.is_a?(Hash))
	  temp = x
	elsif((x.is_a?(NArray))&&(x.rank==2))
	  y = x.shape[1].to_f
	  x = x.shape[0].to_f
	  Basics.checkundef(x, y)
	elsif(Basics.typearray(x) && Basics.typearray(y))
	  Basics.checkundef(x, y)
	elsif(x==nil&&y==nil)
	  Basics.checkundef(1.0, 1.0)
	end
	@param.sync
	@axisparam.sync
	if(temp)
	   @temp_param.set(temp, @param, @backup)
	   @temp_param.sync
	end
	Fig.settrf
	
	DCL.ucyady(cside, jd0, nd)
	if(temp)
	   @backup.sync
	   @temp_param.clear
	end
	nil
      end
      
# by DCL manual
#english
=begin
---Dateaxis.yady(cside, jd0, nd, x=nil, y=nil, temp=nil)
    Plots the coordinate axis for days.

    Arguments
    * cside (String) : Specifies the position to plot the axis.
    * jd0 (Fixnum) : Specifies the date of the position corresponding to 0 in UC.
    * nd (Fixnum) : Specifies the number of days to plot.
    * x, y (Array or NArray) : Arrays for specifying the window.
    * temp (Hash) : temporary parameters

    notes
    (1)UCYADY is called in this method.
    (2)When the window is not specified, the window is specified from x and y if they are specified as arguments.
    (3) The intervals of the tick marks and labels for date are specified using the internal variable 'nday' though the DCL function NUCDAY. When 'nday' is undefined, then an appropriate value is selected out of 0, 1, 2, 5, 10, and 15. (When the value is smaller than 1, only the axis is plotted and no ticks or labels are plotted.) When explicitly specified, the specified value is used. (The initial value is undefined.)
    (4) An array is prepared internally for storing the information for plotting the ticks and days on the labels. The array size restricts the maximum number of ticks and dates (labels) to 100.
    (5) Dateaxis.xady, Dateaxis.yady plots axis using Axis::Lin.xpaxs, Axis::Lin.ypaxs (ISLCT=2), plots tick marks using Axis::Lin.xptmk, Axis::Lin.yptmk (ISLCT=1), and labels of days using Axis::Lin.xplbl, Axis::Lin.yplbl (ISLCT=1). Therefore, the attributes for these component elements of the axes can be inquired/changed using the Axis::Lin module. However, 'icent(zs)' must be 0. 'irotl(zs)' inquires 'irotc(zs)' and uses the value. 'lbtwn' is false. 

=end
#japanese
=begin
---Dateaxis.yady(cside, jd0, nd, x=nil, y=nil, temp=nil)
    ɊւW`. 

    
    * cside (String) : Wꏊw肷.
    * jd0 (Fixnum) : 0ɑꏊ̓tw肷.
    * nd (Fixnum) : 悷w肷.
    * x, y (Array or NArray) : EBhEݒ肷邽߂XAY̔z
    * temp (Hash) : e|p^

    l
    (1)̃\bh UCYADY ĂłB
    (2)EBhEw肵ĂȂƂAƂx,yw肷x,yɃEBhEݒ肵A`B
    (3) ƂɖڐƓt`, DCL ̊֐ NUCDAY ʂăp^ nday ߂. nday sȂ0, 1, 2, 5, 10, 15̒KȒlI΂( 0 ȉ̂Ƃ͎`ŖڐƓt͕`Ȃ); zɗ^Ă΂̒lg(l͕s). 
    (4) ڐƓt`߂̏ێ邽߂ ŔzpӂĂ. ̔z̑傫Ɋւ鐧, `ڐƓt͍ő100ł. 
    (5) Dateaxis.xady, Dateaxis.yady ͂ꂼ Axis::Lin.xpaxs, Axis::Lin.ypaxs (islct = 2)Ŏ`Ă; Axis::Lin.xptmk, Axis::Lin.yptmk (islct = 1)Ŗڐ`Ă; Axis::Lin.xplbl, Axis::Lin.yplbl (islct = 1)œ̃x`Ă. , W̍\vf̑, Axis::Lin ̃p^ɂĎQ/ύX邱Ƃł. , icent(zs)  0 Ƃ; irotl(zs)  irotc(zs)QƂėp; lbtwn  false Ƃ. 

=end
    
      def xamn(cside, jd0, nd, x=nil, y=nil, temp=nil)
	if(x.is_a?(Hash))
	  temp = x
	elsif((x.is_a?(NArray))&&(x.rank==2))
	  y = x.shape[1].to_f
	  x = x.shape[0].to_f
	  Basics.checkundef(x, y)
	elsif(Basics.typearray(x) && Basics.typearray(y))
	  Basics.checkundef(x, y)
	elsif(x==nil&&y==nil)
	  Basics.checkundef(1.0, 1.0)
	end
	@param.sync
	@axisparam.sync
	if(temp)
	   @temp_param.set(temp, @param, @backup)
	   @temp_param.sync
	end
	Fig.settrf
	
	DCL.ucxamn(cside, jd0, nd)
	if(temp)
	   @backup.sync
	   @temp_param.clear
	end
	nil
      end

# by DCL manual
#english
=begin
---Dateaxis.xamn(cside, jd0, nd, x=nil, y=nil, temp=nil)
    Plots the coordinate axis for months.

    Arguments
    * cside (String) : Specifies the position to plot the axis.
    * jd0 (Fixnum) : Specifies the date of the position corresponding to 0 in UC.
    * nd (Fixnum) : Specifies the number of days to plot.
    * x, y (Array or NArray) : Arrays for specifying the window.
    * temp (Hash) : temporary parameters

    Notes
    (1)UCXAMN is called in this method.
    (2)When the window is not specified, the window is specified from x and y if they are specified as arguments.
    (3) The number of letters in the label for month is specified by the internal variable 'nchar' through the DCL function NUCCHR. When 'nchar' is undefined, then an appropriate value is selected out of 0, 1, 2, 3, and 9. (When 'nchar' is 0, only the axis is plotted, and no ticks or labels are plotted.) When explicitly specified, the specified value is used. (The initial value is undefined.) When the value of 'nchar' is negative, the month (label) is expressed with the number of letters corresponding to the absolute value of 'nchar', with the first letter capitalized. 
    (4) An array is prepared internally for storing the information for plotting the ticks and months on the labels. The array size restricts the maximum number of ticks and months (labels) to 50.
    (5) Dateaxis.xamn/Dateaxis.yamn plots axis using Axis::Lin.xpaxs/Axis::Lin.ypaxs (ISLCT=2), plots tick marks using Axis::Lin.xptmk/Axis::Lin.yptmk (ISLCT=1), and labels of months using Axis::Lin.xplbl/Axis::Lin.yplbl (ISLCT=1). Therefore, the attributes for these component elements of the axes can be inquired/changed using the Axis::Lin module. However, 'ICENT(zs)' must be 0. 'irotl(zs)' inquires 'irotc(zs)' and uses the value. 'lbtwn' is true 

=end
#japanese
=begin
---Dateaxis.xamn(cside, jd0, nd, x=nil, y=nil, temp=nil)
    ɊւW`. 

    
    * cside (String) : Wꏊw肷.
    * jd0 (Fixnum) : 0ɑꏊ̓tw肷.
    * nd (Fixnum) : 悷w肷.
    * x, y (Array or NArray) : EBhEݒ肷邽߂XAY̔z
    * temp (Hash) : e|p^

    l
    (1)̃\bh UCXAMN ĂłB
    (2)EBhEw肵ĂȂƂAƂx,yw肷x,yɃEBhEݒ肵A`B
    (3) `, DCL ̊֐ NUCCHR ʂăp^ nchar ߂. nchar sȂ0, 1, 2, 3, 9 ̒KȒlI΂( 0 ȉ̂Ƃ͎`Ŗڐƌ͕`Ȃ); zɗ^Ă΂̒lg(l͕s). nchar ̒l̂Ƃ͂̐Βl̒̌, 2ڈȍ~ɂ.
    (4) ڐƌ`߂̏ێ邽߂ ŔzpӂĂ. ̔z̑傫Ɋւ鐧, `ڐƌ͍ő 50 ł. 
    (5) Dateaxis.xamn, Dateaxis.yamn ͂ꂼ Axis::Lin.xpaxs, Axis::Lin.ypaxs (islct = 2)Ŏ`Ă; Axis::Lin.xptmk, Axis::Lin.yptmk (islct = 2)Ŗڐ`Ă; Axis::Lin.xplbl, Axis::Lin.yplbl (islct = 2)œ̃x`Ă. , W̍\vf̑, Axis::Lin ̃p^ɂĎQ/ύX邱Ƃł. , icent(zs)  0 Ƃ; irotl(zs)  irotc(zs)QƂėp; lbtwn  true Ƃ. 

=end
    
      def yamn(cside, jd0, nd, x=nil, y=nil, temp=nil)
	if(x.is_a?(Hash))
	  temp = x
	elsif((x.is_a?(NArray))&&(x.rank==2))
	  y = x.shape[1].to_f
	  x = x.shape[0].to_f
	  Basics.checkundef(x, y)
	elsif(Basics.typearray(x) && Basics.typearray(y))
	  Basics.checkundef(x, y)
	elsif(x==nil&&y==nil)
	  Basics.checkundef(1.0, 1.0)
	end
	@param.sync
	@axisparam.sync
	if(temp)
	   @temp_param.set(temp, @param, @backup)
	   @temp_param.sync
	end
	Fig.settrf
	
	DCL.ucyamn(cside, jd0, nd)
	if(temp)
	   @backup.sync
	   @temp_param.clear
	end
	nil
      end

# by DCL manual
#english
=begin
---Dateaxis.yamn(cside, jd0, nd, x=nil, y=nil, temp=nil)
    Plots the coordinate axis for months.

    Arguments
    * cside (String) : Specifies the position to plot the axis.
    * jd0 (Fixnum) : Specifies the date of the position corresponding to 0 in UC.
    * nd (Fixnum) : Specifies the number of days to plot.
    * x, y (Array or NArray) : Arrays for specifying the window.
    * temp (Hash) : temporary parameters

    Notes
    (1)UCYAMN is called in this method.
    (2)When the window is not specified, the window is specified from x and y if they are specified as arguments.
    (3) The number of letters in the label for month is specified by the internal variable 'nchar' through the DCL function NUCCHR. When 'nchar' is undefined, then an appropriate value is selected out of 0, 1, 2, 3, and 9. (When 'nchar' is 0, only the axis is plotted, and no ticks or labels are plotted.) When explicitly specified, the specified value is used. (The initial value is undefined.) When the value of 'nchar' is negative, the month (label) is expressed with the number of letters corresponding to the absolute value of 'nchar', with the first letter capitalized. 
    (4) An array is prepared internally for storing the information for plotting the ticks and months on the labels. The array size restricts the maximum number of ticks and months (labels) to 50.
    (5) Dateaxis.xamn/Dateaxis.yamn plots axis using Axis::Lin.xpaxs/Axis::Lin.ypaxs (ISLCT=2), plots tick marks using Axis::Lin.xptmk/Axis::Lin.yptmk (ISLCT=1), and labels of months using Axis::Lin.xplbl/Axis::Lin.yplbl (ISLCT=1). Therefore, the attributes for these component elements of the axes can be inquired/changed using the Axis::Lin module. However, 'ICENT(zs)' must be 0. 'irotl(zs)' inquires 'irotc(zs)' and uses the value. 'lbtwn' is true 

=end
#japanese
=begin
---Dateaxis.yamn(cside, jd0, nd, x=nil, y=nil, temp=nil)
    ɊւW`. 

    
    * cside (String) : Wꏊw肷.
    * jd0 (Fixnum) : 0ɑꏊ̓tw肷.
    * nd (Fixnum) : 悷w肷.
    * x, y (Array or NArray) : EBhEݒ肷邽߂XAY̔z
    * temp (Hash) : e|p^

    l
    (1)̃\bh UCYAMN ĂłB
    (2)EBhEw肵ĂȂƂAƂx,yw肷x,yɃEBhEݒ肵A`B
    (3) `, DCL ̊֐ NUCCHR ʂăp^ nchar ߂. nchar sȂ0, 1, 2, 3, 9 ̒KȒlI΂( 0 ȉ̂Ƃ͎`Ŗڐƌ͕`Ȃ); zɗ^Ă΂̒lg(l͕s). nchar ̒l̂Ƃ͂̐Βl̒̌, 2ڈȍ~ɂ.
    (4) ڐƌ`߂̏ێ邽߂ ŔzpӂĂ. ̔z̑傫Ɋւ鐧, `ڐƌ͍ő 50 ł. 
    (5) Dateaxis.xamn, Dateaxis.yamn ͂ꂼ Axis::Lin.xpaxs, Axis::Lin.ypaxs (islct = 2)Ŏ`Ă; Axis::Lin.xptmk, Axis::Lin.yptmk (islct = 2)Ŗڐ`Ă; Axis::Lin.xplbl, Axis::Lin.yplbl (islct = 2)œ̃x`Ă. , W̍\vf̑, Axis::Lin ̃p^ɂĎQ/ύX邱Ƃł. , icent(zs)  0 Ƃ; irotl(zs)  irotc(zs)QƂėp; lbtwn  true Ƃ. 

=end
    
      def xayr(cside, jd0, nd, x=nil, y=nil, temp=nil)
	if(x.is_a?(Hash))
	  temp = x
	elsif((x.is_a?(NArray))&&(x.rank==2))
	  y = x.shape[1].to_f
	  x = x.shape[0].to_f
	  Basics.checkundef(x, y)
	elsif(Basics.typearray(x) && Basics.typearray(y))
	  Basics.checkundef(x, y)
	elsif(x==nil&&y==nil)
	  Basics.checkundef(1.0, 1.0)
	end
	@param.sync
	@axisparam.sync
	if(temp)
	   @temp_param.set(temp, @param, @backup)
	   @temp_param.sync
	end
	Fig.settrf
	
	DCL.ucxayr(cside, jd0, nd)
	if(temp)
	   @backup.sync
	   @temp_param.clear
	end
	nil
      end

# by DCL manual
#english
=begin
---Dateaxis.xayr(cside, jd0, nd, x=nil, y=nil, temp=nil)
    Plots the coordinate axis for years.

    Arguments
    * cside (String) : Specifies the position to plot the axis.
    * jd0 (Fixnum) : Specifies the date of the position corresponding to 0 in UC.
    * nd (Fixnum) : Specifies the number of days to plot.
    * x, y (Array or NArray) : Arrays for specifying the window.
    * temp (Hash) : temporary parameters

    Notes
    (1)UCXAYR is called in this method.
    (2)When the window is not specified, the window is specified from x and y if they are specified as arguments.
    (3) An array is prepared internally for keeping the information for plotting the ticks and years on the labels. The array size restricts the maximum number of ticks and years (labels) to 50.
    (4) Dateaxis.xayr/Dateaxis.yayr plots axis using Axis::Lin.xpaxs/Axis::Lin.ypaxs (ISLCT=2), plots tick marks using Axis::Lin.xptmk/Axis::Lin.yptmk (ISLCT=1), and labels of years using Axis::Lin.xplbl/Axis::Lin.yplbl (ISLCT=1). Therefore, the attributes for these component elements of the axes can be inquired/changed using the Axis::Lin module. However, 'icent(zs)' must be 0. 'irotl(zs)' inquires 'irotc(zs)' and uses the value. 'lbtwn' is true. 'rsizet2' is multiplied by a factor of 1.5. 

=end
#japanese
=begin
---Dateaxis.xayr(cside, jd0, nd, x=nil, y=nil, temp=nil)
    NɊւW`. 

    
    * cside (String) : Wꏊw肷.
    * jd0 (Fixnum) : 0ɑꏊ̓tw肷.
    * nd (Fixnum) : 悷w肷.
    * x, y (Array or NArray) : EBhEݒ肷邽߂XAY̔z
    * temp (Hash) : e|p^

    l
    (1)̃\bh UCXAYR ĂłB
    (2)EBhEw肵ĂȂƂAƂx,yw肷x,yɃEBhEݒ肵A`B
    (3) ڐƌ`߂̏ێ邽߂ ŔzpӂĂ. ̔z̑傫Ɋւ鐧, `ڐƌ͍ő 50 ł. 
    (4) Dateaxis.xamn, Dateaxis.yamn ͂ꂼ Axis::Lin.xpaxs, Axis::Lin.ypaxs (islct = 2)Ŏ`Ă; Axis::Lin.xptmk, Axis::Lin.yptmk (islct = 2)Ŗڐ`Ă; Axis::Lin.xplbl, Axis::Lin.yplbl (islct = 2)œ̃x`Ă. , W̍\vf̑, Axis::Lin ̃p^ɂĎQ/ύX邱Ƃł. , icent(zs)  0 Ƃ; irotl(zs)  irotc(zs)QƂėp; lbtwn  true Ƃ. 

=end
    
      def yayr(cside, jd0, nd, x=nil, y=nil, temp=nil)
	if(x.is_a?(Hash))
	  temp = x
	elsif((x.is_a?(NArray))&&(x.rank==2))
	  y = x.shape[1].to_f
	  x = x.shape[0].to_f
	  Basics.checkundef(x, y)
	elsif(Basics.typearray(x) && Basics.typearray(y))
	  Basics.checkundef(x, y)
	elsif(x==nil&&y==nil)
	  Basics.checkundef(1.0, 1.0)
	end
	@param.sync
	@axisparam.sync
	if(temp)
	   @temp_param.set(temp, @param, @backup)
	   @temp_param.sync
	end
	Fig.settrf
	
	DCL.ucyayr(cside, jd0, nd)
	if(temp)
	   @backup.sync
	   @temp_param.clear
	end
	nil
      end

# by DCL manual
#english
=begin
---Dateaxis.yayr(cside, jd0, nd, x=nil, y=nil, temp=nil)
    Plots the coordinate axis for years.

    Arguments
    * cside (String) : Specifies the position to plot the axis.
    * jd0 (Fixnum) : Specifies the date of the position corresponding to 0 in UC.
    * nd (Fixnum) : Specifies the number of days to plot.
    * x, y (Array or NArray) : Arrays for specifying the window.
    * temp (Hash) : temporary parameters

    Notes
    (1)UCXAYR is called in this method.
    (2)When the window is not specified, the window is specified from x and y if they are specified as arguments.
    (3) An array is prepared internally for keeping the information for plotting the ticks and years on the labels. The array size restricts the maximum number of ticks and years (labels) to 50.
    (4) Dateaxis.xayr/Dateaxis.yayr plots axis using Axis::Lin.xpaxs/Axis::Lin.ypaxs (ISLCT=2), plots tick marks using Axis::Lin.xptmk/Axis::Lin.yptmk (ISLCT=1), and labels of years using Axis::Lin.xplbl/Axis::Lin.yplbl (ISLCT=1). Therefore, the attributes for these component elements of the axes can be inquired/changed using the Axis::Lin module. However, 'icent(zs)' must be 0. 'irotl(zs)' inquires 'irotc(zs)' and uses the value. 'lbtwn' is true. 'rsizet2' is multiplied by a factor of 1.5. 

=end
#japanese
=begin
---Dateaxis.yayr(cside, jd0, nd, x=nil, y=nil, temp=nil)
    NɊւW`. 

    
    * cside (String) : Wꏊw肷.
    * jd0 (Fixnum) : 0ɑꏊ̓tw肷.
    * nd (Fixnum) : 悷w肷.
    * x, y (Array or NArray) : EBhEݒ肷邽߂XAY̔z
    * temp (Hash) : e|p^

    l
    (1)̃\bh UCYAYR ĂłB
    (2)EBhEw肵ĂȂƂAƂx,yw肷x,yɃEBhEݒ肵A`B
    (3) ڐƌ`߂̏ێ邽߂ ŔzpӂĂ. ̔z̑傫Ɋւ鐧, `ڐƌ͍ő 50 ł. 
    (4) Dateaxis.xamn, Dateaxis.yamn ͂ꂼ Axis::Lin.xpaxs, Axis::Lin.ypaxs (islct = 2)Ŏ`Ă; Axis::Lin.xptmk, Axis::Lin.yptmk (islct = 2)Ŗڐ`Ă; Axis::Lin.xplbl, Axis::Lin.yplbl (islct = 2)œ̃x`Ă. , W̍\vf̑, Axis::Lin ̃p^ɂĎQ/ύX邱Ƃł. , icent(zs)  0 Ƃ; irotl(zs)  irotc(zs)QƂėp; lbtwn  true Ƃ. 

=end

      module_function :param_init
      module_function :help_params
      module_function :xdraw
      module_function :ydraw
      module_function :xady
      module_function :yady
      module_function :set
      module_function :get
      module_function :sync
      module_function :reset
      module_function :xamn
      module_function :yamn
      module_function :xayr
      module_function :yayr
      
      param_init

      @param.keys.each{|i|
	eval "def #{i}; @param['#{i}']; end"
	eval "def #{i}=(val); set('#{i}', val); end"
      }
      
      @param.keys.each{|i|
	eval "module_function :#{i}"
	eval "module_function :#{i}="
      }
    end
  end
end
